from wagtail.core.permission_policies.collections import CollectionOwnershipPermissionPolicy
from wagtail_references import get_reference_model
from wagtail_references.models import Reference


permission_policy = CollectionOwnershipPermissionPolicy(
    get_reference_model(),
    auth_model=Reference,
    owner_field_name='created_by_user'
)



from django.http import HttpResponse
from django.shortcuts import get_object_or_404, redirect, render
from django.urls import reverse
from django.utils.translation import ugettext as _
from django.views.decorators.vary import vary_on_headers
from wagtail.admin import messages
from wagtail.admin.forms.search import SearchForm
from wagtail.admin.utils import PermissionPolicyChecker, permission_denied
from wagtail.core.models import Collection
from wagtail.search import index as search_index
from wagtail.utils.pagination import paginate

from wagtail_references import get_reference_model
from wagtail_references.forms import get_reference_form
from wagtail_references.permissions import permission_policy


permission_checker = PermissionPolicyChecker(permission_policy)


@permission_checker.require_any('add', 'change', 'delete')
@vary_on_headers('X-Requested-With')
def index(request):

    # Get references (filtered by user permission)
    references = permission_policy.instances_user_has_any_permission_for(
        request.user, ['change', 'delete']
    ).order_by('-created_at')

    # Search
    query_string = None
    if 'q' in request.GET:
        form = SearchForm(request.GET, placeholder=_("Search references"))
        if form.is_valid():
            query_string = form.cleaned_data['q']

            references = references.search(query_string)
    else:
        form = SearchForm(placeholder=_("Search references"))

    # Filter by collection
    current_collection = None
    collection_id = request.GET.get('collection_id')
    if collection_id:
        try:
            current_collection = Collection.objects.get(id=collection_id)
            references = references.filter(collection=current_collection)
        except (ValueError, Collection.DoesNotExist):
            pass

    paginator, references = paginate(request, references)

    collections = permission_policy.collections_user_has_any_permission_for(
        request.user, ['add', 'change']
    )
    if len(collections) < 2:
        collections = None

    # Create response
    if request.is_ajax():
        return render(request, 'wagtailreferences/references/results.html', {
            'references': references,
            'query_string': query_string,
            'is_searching': bool(query_string),
        })
    else:
        return render(request, 'wagtailreferences/references/index.html', {
            'references': references,
            'query_string': query_string,
            'is_searching': bool(query_string),
            'search_form': form,
            'collections': collections,
            'current_collection': current_collection,
            'user_can_add': permission_policy.user_has_permission(request.user, 'add'),
        })


@permission_checker.require('change')
def edit(request, reference_id):
    Reference = get_reference_model()
    ReferenceForm = get_reference_form(Reference)

    reference = get_object_or_404(Reference, id=reference_id)

    if not permission_policy.user_has_permission_for_instance(request.user, 'change', reference):
        return permission_denied(request)

    if request.method == 'POST':
        form = ReferenceForm(request.POST, instance=reference, user=request.user)
        if form.is_valid():
            form.save()

            # Reindex the reference
            search_index.insert_or_update_object(reference)

            messages.success(request, _("Reference '{0}' updated.").format(reference.slug), buttons=[
                messages.button(reverse('wagtailreferences:edit', args=(reference.id,)), _('Edit again'))
            ])
            return redirect('wagtailreferences:index')
        else:
            messages.error(request, _("The reference could not be saved due to errors."))
    else:
        form = ReferenceForm(instance=reference, user=request.user)

    return render(request, "wagtailreferences/references/edit.html", {
        'reference': reference,
        'form': form,
        'url_generator_enabled': False,
        'user_can_delete': permission_policy.user_has_permission_for_instance(
            request.user, 'delete', reference
        ),
    })


def preview(request, reference_id):
    reference = get_object_or_404(get_reference_model(), id=reference_id)
    response = HttpResponse(reference.bibtex)
    response['Content-Type'] = 'text/plain'
    return response


@permission_checker.require('delete')
def delete(request, reference_id):
    reference = get_object_or_404(get_reference_model(), id=reference_id)

    if not permission_policy.user_has_permission_for_instance(request.user, 'delete', reference):
        return permission_denied(request)

    if request.method == 'POST':
        reference.delete()
        messages.success(request, _("Reference '{0}' deleted.").format(reference.slug))
        return redirect('wagtailreferences:index')

    return render(request, "wagtailreferences/references/confirm_delete.html", {
        'reference': reference,
    })


@permission_checker.require('add')
def add(request):
    Reference = get_reference_model()
    ReferenceForm = get_reference_form(Reference)

    if request.method == 'POST':
        reference = Reference(created_by_user=request.user)
        form = ReferenceForm(request.POST, request.FILES, instance=reference, user=request.user)
        if form.is_valid():
            form.save()

            # Reindex the image to make sure all tags are indexed
            search_index.insert_or_update_object(reference)

            messages.success(request, _("Reference '{0}' added.").format(reference.slug), buttons=[
                messages.button(reverse('wagtailreferences:edit', args=(reference.id,)), _('Edit'))
            ])
            return redirect('wagtailreferences:index')
        else:
            messages.error(request, _("The reference could not be created due to errors."))
    else:
        form = ReferenceForm(user=request.user)

    return render(request, "wagtailreferences/references/add.html", {
        'form': form,
    })


def usage(request, image_id):
    reference = get_object_or_404(get_reference_model(), id=image_id)

    paginator, used_by = paginate(request, reference.get_usage())

    return render(request, "wagtailreferences/references/usage.html", {
        'reference': reference,
        'used_by': used_by
    })



from django import forms
from django.forms.models import modelform_factory
from django.utils.translation import ugettext as _
from wagtail.admin.forms.collections import (BaseCollectionMemberForm, collection_member_permission_formset_factory)
from wagtail_references.models import Reference
from wagtail_references.permissions import permission_policy as references_permission_policy


def formfield_for_dbfield(db_field, **kwargs):
    return db_field.formfield(**kwargs)


class BaseReferenceForm(BaseCollectionMemberForm):
    permission_policy = references_permission_policy


def get_reference_form(model):
    fields = model.admin_form_fields
    if 'collection' not in fields:
        # force addition of the 'collection' field, because leaving it out can
        # cause dubious results when multiple collections exist (e.g adding the
        # document to the root collection where the user may not have permission) -
        # and when only one collection exists, it will get hidden anyway.
        fields = list(fields) + ['collection']

    return modelform_factory(
        model,
        form=BaseReferenceForm,
        fields=fields,
        formfield_callback=formfield_for_dbfield
    )


class ReferenceInsertionForm(forms.Form):
    """
    Form for selecting parameters of the reference (e.g. format) prior to insertion
    """
    # format = forms.ChoiceField(
    #     choices=[(format.slug, format.label) for format in get_image_formats()],
    #     widget=forms.RadioSelect
    # )
    bibtex = forms.CharField()


# class URLGeneratorForm(forms.Form):
#     filter_method = forms.ChoiceField(
#         label=_("Filter"),
#         choices=(
#             ('original', _("Original size")),
#             ('width', _("Resize to width")),
#             ('height', _("Resize to height")),
#             ('min', _("Resize to min")),
#             ('max', _("Resize to max")),
#             ('fill', _("Resize to fill")),
#         ),
#     )
#     width = forms.IntegerField(label=_("Width"), min_value=0)
#     height = forms.IntegerField(label=_("Height"), min_value=0)
#     closeness = forms.IntegerField(label=_("Closeness"), min_value=0, initial=0)


GroupReferencePermissionFormSet = collection_member_permission_formset_factory(
    Reference,
    [
        ('add_reference', _("Add"), _("Add/edit references you own")),
        ('change_reference', _("Edit"), _("Edit any reference")),
    ],
    'wagtailreferences/permissions/includes/reference_permissions_formset.html'
)



import logging
import bibtexparser
from bibtexparser.bparser import BibTexParser
from bibtexparser.bwriter import BibTexWriter
from bibtexparser.customization import homogenize_latex_encoding, convert_to_unicode
from django.conf import settings
from django.db.models.signals import pre_save
from wagtail_references import get_reference_model


logger = logging.getLogger(__name__)


def pre_save_reference_conversion(instance, **kwargs):
    """ Perform (optional) parsing and conversion of bibtex to ensure validity and force to a uniform type.

    Uses BibtexParser library. Your bibtex may contain accents and specific characters. They are sometimes coded like
    this \'{e} but this is not the correct way. {\'e} is preferred by latex... but for purposes other than using latex,
    you may wish to clean up such entries to unicode e.g. é.

    If WAGTAILREFERENCES_CONVERT_BIBTEX is True (default), clean and convert the input bibtex.
    If 'WAGTAILREFERENCES_ENABLE_UNICODE', is True (default False), convert to unicode, otherwise convert to homogenised LaTeX.

    The entry ID key is always forced to lower case to avoid confusing duplicates.

    :param instance:
    :param kwargs:
    :return:
    """
    parser = BibTexParser()
    if getattr(settings, 'WAGTAILREFERENCES_CONVERT_BIBTEX', True):
        if getattr(settings, 'WAGTAILREFERENCES_ENABLE_UNICODE', False):
            parser.customization = convert_to_unicode
        else:
            parser.customization = homogenize_latex_encoding

    bib_database = bibtexparser.loads(instance.bibtex, parser=parser)

    assert len(bib_database.entries) > 0
    if len(bib_database.entries) > 1:
        bib_database.entries = [bib_database.entries[0]]
        logger.warning('More than one entry in submitted bibtex string. Removing all but the first.')

    bib_database.entries[0]['ID'] = bib_database.entries[0]['ID'].lower()

    writer = BibTexWriter()
    writer.indent = '    '

    instance.slug = bib_database.entries[0]['ID']
    instance.bibtype = bib_database.entries[0]['ENTRYTYPE']
    instance.bibtex = bibtexparser.dumps(bib_database, writer=writer)


def register_signal_handlers():
    pre_save.connect(pre_save_reference_conversion, sender=get_reference_model())



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class ReferencesAppConfig(AppConfig):
    name = 'wagtail_references'
    label = 'wagtail_references'
    verbose_name = _('Wagtail References')

    def ready(self):
        from wagtail_references.signal_handlers import register_signal_handlers
        register_signal_handlers()



from django.forms.fields import CharField
from django.utils.translation import ugettext_lazy as _


# TODO probably unused. Delete.


class ReferenceField(CharField):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Help text
        self.help_text = _('A bibtex reference')

        # Error messages
        self.error_messages['invalid_reference'] = _('Not a valid bibtex reference.')

        self.error_messages['invalid_image_known_format'] = _(
            "Not a valid %s image."
        )



from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from wagtail.core.models import CollectionMember
from wagtail.search import index
from wagtail.search.queryset import SearchableQuerySetMixin
from wagtail.snippets.models import register_snippet


class ReferenceQuerySet(SearchableQuerySetMixin, models.QuerySet):
    pass


class AbstractReference(CollectionMember, index.Indexed, models.Model):
    slug = models.CharField(max_length=255, unique=True, verbose_name=_('slug'), help_text=_('A short key to cite the reference by. Determined from the BibTeX entry key. Must be unique.'))
    bibtex = models.TextField(help_text=_('The reference, in bibtex format.'))
    bibtype = models.CharField(max_length=255, verbose_name=_('Bibliography entry type'), default='article', help_text=_('The entry type, detected from the BibTeX entry.'))
    created_at = models.DateTimeField(verbose_name=_('created at'), auto_now_add=True, db_index=True)
    created_by_user = models.ForeignKey(
        settings.AUTH_USER_MODEL, verbose_name=_('created by user'),
        null=True, blank=True, editable=False, on_delete=models.SET_NULL
    )

    objects = ReferenceQuerySet.as_manager()

    search_fields = CollectionMember.search_fields + [
        index.SearchField('slug', partial_match=True, boost=10),
        index.SearchField('bibtex', partial_match=True, boost=10),
        index.AutocompleteField('slug'),
        index.FilterField('bibtype'),
        index.FilterField('created_by_user'),
    ]

    def __str__(self):
        return self.bibtex

    def is_editable_by_user(self, user):
        from wagtail.images.permissions import permission_policy
        return permission_policy.user_has_permission_for_instance(user, 'change', self)

    class Meta:
        abstract = True


@register_snippet
class Reference(AbstractReference):
    admin_form_fields = (
        'bibtex',
        'collection',
    )

    class Meta:
        verbose_name = _('BibTeX Reference')
        verbose_name_plural = _('BibTeX References')



from rest_framework import serializers
from .models import Reference


class ReferenceSerializer(serializers.ModelSerializer):

    class Meta:

        model = Reference

        fields = (
            'slug',
            'bibtype',
            'bibtex'
        )

        read_only_fields = (
            'slug',
            'bibtype',
        )

        depth = 1



import base64
import hashlib
import hmac

from django.conf import settings
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.urls import reverse
from django.utils.encoding import force_text
from django.views.generic import View

from wagtail_references import get_reference_model


def generate_signature(reference_id, filter_spec, key=None):
    if key is None:
        key = settings.SECRET_KEY

    # Key must be a bytes object
    if isinstance(key, str):
        key = key.encode()

    # Based on libthumbor hmac generation
    # https://github.com/thumbor/libthumbor/blob/b19dc58cf84787e08c8e397ab322e86268bb4345/libthumbor/crypto.py#L50
    url = '{}/{}/'.format(reference_id, filter_spec)
    return force_text(base64.urlsafe_b64encode(hmac.new(key, url.encode(), hashlib.sha1).digest()))


def verify_signature(signature, reference_id, filter_spec, key=None):
    return force_text(signature) == generate_signature(reference_id, filter_spec, key=key)


def generate_reference_url(reference, filter_spec, viewname='wagtailreferences_serve', key=None):
    signature = generate_signature(reference.id, filter_spec, key)
    url = reverse(viewname, args=(signature, reference.id, filter_spec))
    url += reference.file.name[len('original_references/'):]
    return url


class ServeView(View):
    model = get_reference_model()
    key = None

    def get(self, request, reference_id):

        reference = get_object_or_404(self.model, id=reference_id)

        return HttpResponse(reference.bibtex, content_type='text/plain')


serve = ServeView.as_view()



from django.utils.functional import cached_property

from wagtail.core.blocks import ChooserBlock

# from .shortcuts import get_rendition_or_not_found


class ReferenceChooserBlock(ChooserBlock):
    @cached_property
    def target_model(self):
        from wagtail_references import get_reference_model
        return get_reference_model()

    # @cached_property
    # def widget(self):
    #     from wagtail_references.widgets import AdminReferenceChooser
    #     return AdminReferenceChooser

    # def render_basic(self, value, context=None):
    #     if value:
    #                 return get_rendition_or_not_found(value, 'original').img_tag()
    #             else:
    #         return ''

    class Meta:
        icon = "list-ol"



# Generated by Django 2.1.4 on 2018-12-19 11:15

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import wagtail.core.models
import wagtail.search.index


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0040_page_draft_title'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Reference',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.CharField(help_text='A short key to cite the reference by. Determined from the BibTeX entry key. Must be unique.', max_length=255, unique=True, verbose_name='slug')),
                ('bibtex', models.TextField(help_text='The reference, in bibtex format.')),
                ('bibtype', models.CharField(default='article', help_text='The entry type, detected from the BibTeX entry.', max_length=255, verbose_name='Bibliography entry type')),
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='created at')),
                ('collection', models.ForeignKey(default=wagtail.core.models.get_root_collection_id, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='wagtailcore.Collection', verbose_name='collection')),
                ('created_by_user', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='created by user')),
            ],
            options={
                'verbose_name': 'BibTeX Reference',
                'verbose_name_plural': 'BibTeX References',
            },
            bases=(wagtail.search.index.Indexed, models.Model),
        ),
    ]



from django.conf.urls import url

from wagtail_references.views import references, chooser


app_name = 'wagtail_references'


urlpatterns = [
    url(r'^$', references.index, name='index'),
    url(r'^(\d+)/$', references.edit, name='edit'),
    url(r'^(\d+)/delete/$', references.delete, name='delete'),
    url(r'^(\d+)/preview/(.*)/$', references.preview, name='preview'),
    url(r'^add/$', references.add, name='add'),
    url(r'^usage/(\d+)/$', references.usage, name='reference_usage'),
    url(r'^chooser/$', chooser.chooser, name='chooser'),
    url(r'^chooser/(\d+)/$', chooser.reference_chosen, name='reference_chosen'),
]



from django.conf.urls import include, url
from django.urls import reverse
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ungettext

from wagtail.admin.menu import MenuItem
from wagtail.admin.search import SearchArea
from wagtail.admin.site_summary import SummaryItem
from wagtail.core import hooks


from wagtail_references import admin_urls, get_reference_model
# from references.api.admin.endpoints import ReferencesAdminAPIEndpoint
from wagtail_references.forms import GroupReferencePermissionFormSet
from wagtail_references.permissions import permission_policy


@hooks.register('register_admin_urls')
def register_admin_urls():
    return [
        url(r'^references/', include(admin_urls, namespace='wagtailreferences')),
    ]


# @hooks.register('construct_admin_api')
# def construct_admin_api(router):
#     router.register_endpoint('references', ReferencesAdminAPIEndpoint)


class ReferencesMenuItem(MenuItem):
    def is_shown(self, request):
        return permission_policy.user_has_any_permission(
            request.user, ['add', 'change', 'delete']
        )


@hooks.register('register_admin_menu_item')
def register_images_menu_item():
    return ReferencesMenuItem(
        _('References'), reverse('wagtail_references:index'),
        name='references', classnames='icon icon-list-ol', order=300
    )


@hooks.register('insert_editor_js')
def editor_js():
    return format_html(
        """
        <script>
            window.chooserUrls.referenceChooser = '{0}';
        </script>
        """,
        reverse('wagtail_references:chooser')
    )


class ReferencesSummaryItem(SummaryItem):
    order = 200
    template = 'wagtailreferences/homepage/site_summary_references.html'

    def get_context(self):
        return {
            'total_references': get_reference_model().objects.count(),
        }

    def is_shown(self):
        return permission_policy.user_has_any_permission(
            self.request.user, ['add', 'change', 'delete']
        )


@hooks.register('construct_homepage_summary_items')
def add_references_summary_item(request, items):
    items.append(ReferencesSummaryItem(request))


class ReferencesSearchArea(SearchArea):
    def is_shown(self, request):
        return permission_policy.user_has_any_permission(
            request.user, ['add', 'change', 'delete']
        )


@hooks.register('register_admin_search_area')
def register_references_search_area():
    return ReferencesSearchArea(
        _('References'), reverse('wagtail_references:index'),
        name='references',
        classnames='icon icon-list-ol',
        order=200)


@hooks.register('register_group_permission_panel')
def register_reference_permissions_panel():
    return GroupReferencePermissionFormSet


@hooks.register('describe_collection_contents')
def describe_collection_docs(collection):
    references_count = get_reference_model().objects.filter(collection=collection).count()
    if references_count:
        url = reverse('wagtail_references:index') + ('?collection_id=%d' % collection.id)
        return {
            'count': references_count,
            'count_text': ungettext(
                "%(count)s reference",
                "%(count)s references",
                references_count
            ) % {'count': references_count},
            'url': url,
        }



from django.template.loader import render_to_string

from wagtail.admin.compare import ForeignObjectComparison
from wagtail.admin.edit_handlers import BaseChooserPanel


class ReferenceChooserPanel(BaseChooserPanel):
    object_type_name = "reference"

    # def widget_overrides(self):
    #     TODO reinstate if widget needs to be overridden - see how the wagtailimages widget works
    #     from .widgets import AdminReferenceChooser
    #     return {self.field_name: AdminReferenceChooser}

    def get_comparison_class(self):
        return ReferenceFieldComparison


class ReferenceFieldComparison(ForeignObjectComparison):
    def htmldiff(self):
        reference_a, reference_b = self.get_objects()

        return render_to_string("wagtailreferences/widgets/compare.html", {
            'reference_a': reference_a,
            'reference_b': reference_b,
        })



from django.shortcuts import get_object_or_404, render
from django.urls import reverse
from django.utils.translation import ugettext as _
from wagtail.admin.forms.search import SearchForm
from wagtail.admin.modal_workflow import render_modal_workflow
from wagtail.admin.utils import PermissionPolicyChecker
from wagtail.core import hooks
from wagtail.core.models import Collection
from wagtail.utils.pagination import paginate

from wagtail_references import get_reference_model
from wagtail_references.forms import ReferenceInsertionForm, get_reference_form
from wagtail_references.permissions import permission_policy


permission_checker = PermissionPolicyChecker(permission_policy)


def get_chooser_js_data():
    """construct context variables needed by the chooser JS"""
    return {
        'step': 'chooser',
        'error_label': _("Server Error"),
        'error_message': _("Report this error to your webmaster with the following information:"),
    }


def get_reference_result_data(reference):
    """
    helper function: given a reference, return the json data to pass back to the
    reference chooser panel
    """
    return {
        'id': reference.id,
        'edit_link': reverse('wagtailreferences:edit', args=(reference.id,)),
        'slug': reference.slug,
        'preview': {
            'bibtex': reference.bibtex
        }
    }


def get_chooser_context(request):
    """Helper function to return common template context variables for the main chooser view"""

    collections = Collection.objects.all()
    if len(collections) < 2:
        collections = None
    else:
        collections = Collection.order_for_display(collections)

    return {
        'searchform': SearchForm(),
        'is_searching': False,
        'query_string': None,
        'will_select_format': request.GET.get('select_format'),
        'collections': collections,
    }


def chooser(request):
    Reference = get_reference_model()

    if permission_policy.user_has_permission(request.user, 'add'):
        ReferenceForm = get_reference_form(Reference)
        uploadform = ReferenceForm(user=request.user)
    else:
        uploadform = None

    # TODO add an author field and order by that
    references = Reference.objects.order_by('-created_at')

    # allow hooks to modify the queryset
    for hook in hooks.get_hooks('construct_reference_chooser_queryset'):
        references = hook(references, request)

    if (
        'q' in request.GET or 'p' in request.GET or 'tag' in request.GET
        or 'collection_id' in request.GET
    ):
        # this request is triggered from search, pagination or 'popular tags';
        # we will just render the results.html fragment
        collection_id = request.GET.get('collection_id')
        if collection_id:
            references = references.filter(collection=collection_id)

        searchform = SearchForm(request.GET)
        if searchform.is_valid():
            q = searchform.cleaned_data['q']

            references = references.search(q)
            is_searching = True
        else:
            is_searching = False
            q = None

        paginator, images = paginate(request, references, per_page=50)
        return render(request, "wagtailreferences/chooser/results.html", {
            'references': references,
            'is_searching': is_searching,
            'query_string': q,
            'will_select_format': request.GET.get('select_format')
        })
    else:
        paginator, images = paginate(request, references, per_page=50)
        context = get_chooser_context(request)
        context.update({
            'references': references,
            'uploadform': uploadform,
        })
        return render_modal_workflow(
            request, 'wagtailreferences/chooser/chooser.html', None, context
        )


def reference_chosen(request, reference_id):
    reference = get_object_or_404(get_reference_model(), id=reference_id)

    return render_modal_workflow(
        request, None, None,
        None, json_data={'step': 'reference_chosen', 'result': get_reference_result_data(reference)}
    )


def chooser_select_format(request, reference_id):
    reference = get_object_or_404(get_reference_model(), id=reference_id)

    if request.method == 'POST':
        form = ReferenceInsertionForm(request.POST)
        if form.is_valid():
            reference_data = {
                'id': reference.id,
                'slug': reference.slug,
                'edit_link': reverse('wagtailreferences:edit', args=(reference.id,)),
                'bibtex': reference.bibtex
            }

            return render_modal_workflow(
                request, None, None,
                None, json_data={'step': 'reference_chosen', 'result': reference_data}
            )
    else:
        initial = {}
        initial.update(request.GET.dict())
        form = ReferenceInsertionForm(initial=initial)

    return render_modal_workflow(
        request, 'wagtailreferences/chooser/select_format.html', None,
        {'reference': reference, 'form': form}, json_data={'step': 'select_format'}
    )
